"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
///<reference types="typescript/lib/lib.es6"/>
var http_1 = require("@angular/http");
require("rxjs/add/operator/map");
var parameters_1 = require("../decorators/parameters");
function methodBuilder(method) {
    return function (url) {
        return function (target, propertyKey, descriptor) {
            var pPath = target[propertyKey + "_Path_parameters"];
            var pQuery = target[propertyKey + "_Query_parameters"];
            var pBody = target[propertyKey + "_Body_parameters"];
            var pPlainBody = target[propertyKey + "_PlainBody_parameters"];
            var pHeader = target[propertyKey + "_Header_parameters"];
            descriptor.value = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                // Body
                var body = null;
                if (pBody) {
                    if (pBody.length > 1) {
                        throw new Error('Only one @Body is allowed');
                    }
                    var value = args[pBody[0].parameterIndex];
                    if (value === undefined && pBody[0].value !== undefined) {
                        value = pBody[0].value;
                    }
                    body = JSON.stringify(value);
                }
                if (pPlainBody) {
                    if (pPlainBody.length > 1) {
                        throw new Error('Only one @Body is allowed');
                    }
                    var value = args[pPlainBody[0].parameterIndex];
                    if (value === undefined && pPlainBody[0].value !== undefined) {
                        value = pPlainBody[0].value;
                    }
                    body = value;
                }
                // Path
                var resUrl = url;
                if (pPath) {
                    for (var k in pPath) {
                        if (pPath.hasOwnProperty(k)) {
                            var value_1 = args[pPath[k].parameterIndex];
                            if (value_1 === undefined && pPath[k].value !== undefined) {
                                value_1 = pPath[k].value;
                            }
                            if (value_1 !== undefined && value_1 !== null) {
                                resUrl = resUrl.replace('{' + pPath[k].key + '}', value_1);
                            }
                            else {
                                throw new Error('Missing path variable \'' + pPath[k].key + '\' in url ' + url);
                            }
                        }
                    }
                }
                if (this.getBaseUrl() !== null) {
                    var baseUrl = this.getBaseUrl();
                    if (baseUrl.indexOf('/') === baseUrl.length - 1 && resUrl.indexOf('/') === 0) {
                        baseUrl = baseUrl.substring(0, 1);
                    }
                    resUrl = baseUrl + resUrl;
                }
                // Query
                var search = new http_1.URLSearchParams();
                if (pQuery) {
                    pQuery
                        .filter(function (p) { return args[p.parameterIndex] !== undefined || p.value !== undefined; }) // filter out optional parameters
                        .forEach(function (p) {
                        var key = p.key;
                        var value = args[p.parameterIndex];
                        if (value === undefined && p.value !== undefined) {
                            value = p.value;
                        }
                        // if the value is a instance of Object, we stringify it
                        if (Array.isArray(value)) {
                            switch (p.format) {
                                case parameters_1.Format.CSV:
                                    value = value.join(',');
                                    break;
                                case parameters_1.Format.SSV:
                                    value = value.join(' ');
                                    break;
                                case parameters_1.Format.TSV:
                                    value = value.join('\t');
                                    break;
                                case parameters_1.Format.PIPES:
                                    value = value.join('|');
                                    break;
                                case parameters_1.Format.MULTI:
                                    value = value;
                                    break;
                                default:
                                    value = value.join(',');
                            }
                        }
                        else if (value instanceof Object) {
                            value = JSON.stringify(value);
                        }
                        if (Array.isArray(value)) {
                            value.forEach(function (v) { return search.append(key, v); });
                        }
                        else {
                            search.set(key, value);
                        }
                    });
                }
                // Headers
                // set class default headers
                var headers = new http_1.Headers(this.getDefaultHeaders());
                // set method specific headers
                for (var k in descriptor.headers) {
                    if (descriptor.headers.hasOwnProperty(k)) {
                        headers.append(k, descriptor.headers[k]);
                    }
                }
                // set parameter specific headers
                if (pHeader) {
                    for (var k in pHeader) {
                        if (pHeader.hasOwnProperty(k)) {
                            var value_2 = args[pHeader[k].parameterIndex];
                            if (value_2 === undefined && pHeader[k].value !== undefined) {
                                value_2 = pHeader[k].value;
                            }
                            if (Array.isArray(value_2)) {
                                switch (pHeader[k].format) {
                                    case parameters_1.Format.CSV:
                                        value_2 = value_2.join(',');
                                        break;
                                    case parameters_1.Format.SSV:
                                        value_2 = value_2.join(' ');
                                        break;
                                    case parameters_1.Format.TSV:
                                        value_2 = value_2.join('\t');
                                        break;
                                    case parameters_1.Format.PIPES:
                                        value_2 = value_2.join('|');
                                        break;
                                    case parameters_1.Format.MULTI:
                                        value_2 = value_2;
                                        break;
                                    default:
                                        value_2 = value_2.join(',');
                                }
                            }
                            if (Array.isArray(value_2)) {
                                value_2.forEach(function (v) { return headers.append(pHeader[k].key, v); });
                            }
                            else {
                                headers.append(pHeader[k].key, value_2);
                            }
                        }
                    }
                }
                // Request options
                var options = new http_1.RequestOptions({
                    method: method,
                    url: resUrl,
                    headers: headers,
                    body: body,
                    search: search,
                    withCredentials: true
                });
                var req = new http_1.Request(options);
                // intercept the request
                this.requestInterceptor(req);
                // make the request and store the observable for later transformation
                var observable = this.http.request(req);
                // transform the observable in accordance to the @Produces decorator
                if (descriptor.mime) {
                    observable = observable.map(descriptor.mime);
                }
                if (descriptor.timeout) {
                    descriptor.timeout.forEach(function (timeout) {
                        observable = observable.timeout(timeout);
                    });
                }
                if (descriptor.mappers) {
                    descriptor.mappers.forEach(function (mapper) {
                        observable = observable.map(mapper);
                    });
                }
                if (descriptor.emitters) {
                    descriptor.emitters.forEach(function (handler) {
                        observable = handler(observable);
                    });
                }
                // intercept the response
                observable = this.responseInterceptor(observable);
                return observable;
            };
            return descriptor;
        };
    };
}
exports.methodBuilder = methodBuilder;
