"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * collection Formats
 */
exports.Format = {
    /**
     *  comma separated values foo,bar.
     */
    CSV: 'CSV',
    /**
     *  space separated values foo bar.
     */
    SSV: 'SSV',
    /**
     *  tab separated values foo\tbar.
     */
    TSV: 'TSV',
    /**
     *  pipe separated values foo|bar.
     */
    PIPES: 'PIPES',
    /**
     *  corresponds to multiple parameter instances instead of multiple values for a single instance foo=bar&foo=baz.
     *  This is valid only for parameters in "query" or "formData".
     */
    MULTI: 'MULTI',
};
function paramBuilder(paramName) {
    return function (name, value) {
        return function (target, propertyKey, parameterIndex) {
            var format;
            if (value) {
                if (typeof (value) === 'object') {
                    if (value.value !== undefined && value.value !== null) {
                        value = value.value;
                    }
                    if (value.format !== undefined && value.format !== null) {
                        if (exports.Format[value.format] !== undefined) {
                            format = value.format;
                        }
                        else {
                            throw new Error('Unknown Collection Format: \'' + value.format + '\'');
                        }
                    }
                }
            }
            var metadataKey = propertyKey + "_" + paramName + "_parameters";
            var paramObj = {
                key: name,
                parameterIndex: parameterIndex,
                value: value,
                format: format
            };
            if (Array.isArray(target[metadataKey])) {
                target[metadataKey].push(paramObj);
            }
            else {
                target[metadataKey] = [paramObj];
            }
        };
    };
}
exports.paramBuilder = paramBuilder;
/**
 * Path variable of a method's url, type: string
 * @param {string} key - path key to bind value
 */
exports.Path = paramBuilder('Path');
/**
 * Query value of a method's url, type: string
 * @param {string} key - query key to bind value
 */
exports.Query = paramBuilder('Query');
/**
 * Body of a REST method, type: key-value pair object
 * Only one body per method!
 */
exports.Body = paramBuilder('Body')('Body');
/**
 * Body of a REST method, type: key-value pair string separated by '&'
 * Only one body per method!
 */
exports.PlainBody = paramBuilder('PlainBody')('PlainBody');
/**
 * Custom header of a REST method, type: string
 * @param {string} key - header key to bind value
 */
exports.Header = paramBuilder('Header');
